﻿Imports Autodesk.AutoCAD.Runtime
Imports Autodesk.AutoCAD.DatabaseServices
Imports Autodesk.AutoCAD.Geometry
Imports Autodesk.AutoCAD.ApplicationServices
Imports Autodesk.AutoCAD.EditorInput


Namespace HorizontalAttributes

    Public Class Commands

        ' Class variable to store the instance of our overrule
        Private Shared myOverrule As KeepStraightOverrule
        ' Registered Aplication Id for Xdata
        Const mXdataName As String = "ADSK_ATTRIBUTE_ZERO_OVERRULE"


        <CommandMethod("KeepStraight")>
        Public Shared Sub ImplementOverrule()

            Dim doc As Document =
                    Application.DocumentManager.MdiActiveDocument
            Dim ed As Editor = doc.Editor

            ' Select a block reference

            Dim opts As New PromptEntityOptions(
              vbLf & "Select a block reference:")
            opts.SetRejectMessage(vbLf & "Must be block reference...")
            opts.AddAllowedClass(GetType(BlockReference), True)

            Dim res As PromptEntityResult = ed.GetEntity(opts)

            If res.Status <> PromptStatus.OK Then
                Exit Sub
            End If

            Dim db As Database = doc.Database

            AddRegAppId(db)

            Using trans As Transaction =
                  db.TransactionManager.StartTransaction

                ' Open the BlockReference for read.
                ' We know its a BlockReference because we set a filter in 
                '  our PromptEntityOptions
                Dim blkRef As BlockReference =
                        trans.GetObject(res.ObjectId, OpenMode.ForRead)
                ' Record the ObjectIds of all AttributeReferences 
                '  attached to the BlockReference.
                Dim attRefColl As AttributeCollection =
                                                  blkRef.AttributeCollection

                ' Iterate through  ObjectIds of all AttributeReferences 
                '  attached to the BlockReference, opening each
                '  AttributeReference and adding xdata to it.
                For Each objId As ObjectId In attRefColl
                    Dim attRef As AttributeReference =
                                      trans.GetObject(objId, OpenMode.ForWrite)
                    ' Create new xdata containing the overrule filter name
                    Using resBuf As New ResultBuffer(
                      New TypedValue(
                       DxfCode.ExtendedDataRegAppName, mXdataName),
                      New TypedValue(
                       DxfCode.ExtendedDataAsciiString, "Dummy text"))

                        ' Add the xdata
                        attRef.XData = resBuf
                    End Using
                Next
                trans.Commit()

            End Using

            ' Create and register our overrule and turn overruling on.
            ActivateOverrule()
        End Sub

        <CommandMethod("DontKeepStraight")>
        Public Shared Sub RemoveXdata()

            Dim doc As Document =
                  Application.DocumentManager.MdiActiveDocument
            Dim ed As Editor = doc.Editor

            ' Select a block reference

            Dim opts As New PromptEntityOptions(
              vbLf & "Select a block reference:")
            opts.SetRejectMessage(vbLf & "Must be block reference...")
            opts.AddAllowedClass(GetType(BlockReference), True)

            Dim res As PromptEntityResult = ed.GetEntity(opts)

            If res.Status <> PromptStatus.OK Then
                Exit Sub
            End If

            Dim db As Database = doc.Database

            AddRegAppId(db)

            Using trans As Transaction =
                  db.TransactionManager.StartTransaction

                ' Open the BlockReference for read.
                ' We know its a BlockReference because we set a filter in 
                '  our PromptEntityOptions
                Dim blkRef As BlockReference =
                        trans.GetObject(res.ObjectId, OpenMode.ForRead)
                ' Record the ObjectIds of all AttributeReferences 
                '  attached to the BlockReference.
                Dim attRefColl As AttributeCollection =
                                                  blkRef.AttributeCollection

                ' Iterate through  ObjectIds of all AttributeReferences 
                '  attached to the BlockReference, opening each
                '  AttributeReference and adding xdata to it.
                For Each objId As ObjectId In attRefColl
                    Dim attRef As AttributeReference =
                                trans.GetObject(objId, OpenMode.ForWrite)

                    ' Create new xdata containing the overrule filter name
                    Using resBuf As New ResultBuffer(
                      New TypedValue(
                       DxfCode.ExtendedDataRegAppName, mXdataName))
                        ' Add the xdata.
                        ' Because we leave this blank except for the regappid,
                        '  it erases any Xdata we previously added.
                        attRef.XData = resBuf
                    End Using
                Next
                trans.Commit()

            End Using

        End Sub

        <CommandMethod("ActivateOverrule")>
        Public Shared Sub ActivateOverrule()

            'We only want to create our overrule instance once, 
            ' so we check if it already exists before we create it
            ' (i.e. this may be the 2nd time we've run the command)
            If myOverrule Is Nothing Then
                'Instantiate our overrule class
                myOverrule = New KeepStraightOverrule
                'Register the overrule
                Overrule.AddOverrule(
                  RXClass.GetClass(GetType(AttributeReference)),
                  myOverrule, False)
            End If

            'Specify which Attributes will be overruled
            myOverrule.SetXDataFilter(mXdataName)
            'Make sure overruling is turned on so our overrule works
            Overrule.Overruling = True

        End Sub

        Private Shared Sub AddRegAppId(ByVal db As Database)

            Using trans As Transaction =
              db.TransactionManager.StartTransaction()

                ' First create our RegAppId (if it doesn't already exist)
                Dim appTbl As RegAppTable =
                  trans.GetObject(db.RegAppTableId, OpenMode.ForRead)

                If Not appTbl.Has(mXdataName) Then
                    Dim appTblRec As New RegAppTableRecord
                    appTbl.UpgradeOpen()
                    appTblRec.Name = mXdataName
                    appTbl.Add(appTblRec)
                    trans.AddNewlyCreatedDBObject(appTblRec, True)
                End If
                trans.Commit()
            End Using
        End Sub

    End Class

    'Our custom overrule class derived from TransformOverrule
    Public Class KeepStraightOverrule
        Inherits TransformOverrule

        'We want to change how an AttributeReference responds to being
        ' transformed (moved, rotated, etc.), so we override its
        ' standard TransformBy function.
        Public Overrides Sub TransformBy(ByVal entity As Entity,
                                         ByVal transform As Matrix3d)

            'Call the normal TransformBy function for the attribute
            ' reference we're overruling.
            MyBase.TransformBy(entity, transform)
            'We know entity must be an AttributeReference because 
            ' that is the only entity we registered the overrule for.
            Dim attRef As AttributeReference = entity
            'Set rotation of attribute reference to 0 (horizontal)
            attRef.Rotation = 0.0

        End Sub
    End Class
End Namespace
